@@ -17,11 +17,12 @@
 from collections import Counter
 import logging
 import os
-import multiprocessing
 import time
 import subprocess
+import six
 import sys
 
+import multiprocessing
 from concurrent.futures import (
     ProcessPoolExecutor,
     as_completed)
@@ -37,9 +38,9 @@ from c7n.credentials import assumed_session, SessionFactory
 from c7n.executor import MainThreadExecutor
 from c7n.config import Config
 from c7n.policy import PolicyCollection
+from c7n.provider import get_resource_class
 from c7n.reports.csvout import Formatter, fs_record_set
 from c7n.resources import load_resources
-from c7n.manager import resources as resource_registry
 from c7n.utils import CONN_CACHE, dumps
 
 from c7n_org.utils import environ, account_tags
@@ -47,10 +48,13 @@ from c7n.utils import UnicodeWriter
 
 log = logging.getLogger('c7n_org')
 
-# On OSX High Sierra Workaround
-# https://github.com/ansible/ansible/issues/32499
-if sys.platform == 'darwin':
-    os.environ['OBJC_DISABLE_INITIALIZE_FORK_SAFETY'] = 'YES'
+# Workaround OSX issue, note this exists for py2 but there
+# isn't anything we can do in that case.
+# https://bugs.python.org/issue33725
+if sys.platform == 'darwin' and (
+        sys.version_info.major > 3 and sys.version_info.minor > 4):
+    multiprocessing.set_start_method('spawn')
+
 
 WORKER_COUNT = int(
     os.environ.get('C7N_ORG_PARALLEL', multiprocessing.cpu_count() * 4))
@@ -78,6 +82,7 @@ CONFIG_SCHEMA = {
                     {'type': 'array', 'items': {'type': 'string'}},
                     {'type': 'string', 'minLength': 3}]},
                 'external_id': {'type': 'string'},
+                'vars': {'type': 'object'},
             }
         },
         'subscription': {
@@ -88,6 +93,7 @@ CONFIG_SCHEMA = {
                 'subscription_id': {'type': 'string'},
                 'tags': {'type': 'array', 'items': {'type': 'string'}},
                 'name': {'type': 'string'},
+                'vars': {'type': 'object'},
             }
         },
         'project': {
@@ -98,6 +104,7 @@ CONFIG_SCHEMA = {
                 'project_id': {'type': 'string'},
                 'tags': {'type': 'array', 'items': {'type': 'string'}},
                 'name': {'type': 'string'},
+                'vars': {'type': 'object'},
             }
         },
     },
@@ -158,6 +165,7 @@ def init(config, use, debug, verbose, accounts, tags, policies, resource=None, p
         level=level,
         format="%(asctime)s: %(name)s:%(levelname)s %(message)s")
 
+    logging.getLogger().setLevel(level)
     logging.getLogger('botocore').setLevel(logging.ERROR)
     logging.getLogger('s3transfer').setLevel(logging.WARNING)
     logging.getLogger('custodian.s3').setLevel(logging.ERROR)
@@ -169,7 +177,7 @@ def init(config, use, debug, verbose, accounts, tags, policies, resource=None, p
         if isinstance(h, logging.StreamHandler):
             h.addFilter(LogFilter())
 
-    with open(config) as fh:
+    with open(config, 'rb') as fh:
         accounts_config = yaml.safe_load(fh.read())
         jsonschema.validate(accounts_config, CONFIG_SCHEMA)
 
@@ -189,9 +197,10 @@ def init(config, use, debug, verbose, accounts, tags, policies, resource=None, p
     return accounts_config, custodian_config, executor
 
 
-def resolve_regions(regions, partition='aws'):
+def resolve_regions(regions):
     if 'all' in regions:
-        return boto3.Session().get_available_regions('ec2', partition)
+        client = boto3.client('ec2')
+        return [region['RegionName'] for region in client.describe_regions()['Regions']]
     if not regions:
         return ('us-east-1', 'us-west-2')
     return regions
@@ -199,9 +208,22 @@ def resolve_regions(regions, partition='aws'):
 
 def get_session(account, session_name, region):
     if account.get('role'):
-        return assumed_session(
-            account['role'], session_name, region=region,
-            external_id=account.get('external_id'))
+        roles = account['role']
+        if isinstance(roles, six.string_types):
+            roles = [roles]
+        s = None
+        for r in roles:
+            try:
+                s = assumed_session(
+                    r, session_name, region=region,
+                    external_id=account.get('external_id'),
+                    session=s)
+            except ClientError as e:
+                log.error(
+                    "unable to obtain credentials for account:%s role:%s error:%s",
+                    account['name'], r, e)
+                raise
+        return s
     elif account.get('profile'):
         return SessionFactory(region, account['profile'])()
     else:
@@ -353,8 +375,8 @@ def report(config, output, use, output_dir, accounts,
     prefix_fields = OrderedDict(
         (('Account', 'account'), ('Region', 'region'), ('Policy', 'policy')))
     config = Config.empty()
-    factory = resource_registry.get(list(resource_types)[0])
 
+    factory = get_resource_class(list(resource_types)[0])
     formatter = Formatter(
         factory.resource_type,
         extra_fields=field,
@@ -369,21 +391,24 @@ def report(config, output, use, output_dir, accounts,
     writer.writerows(rows)
 
 
-def run_account_script(account, region, output_dir, debug, script_args):
-    try:
-        session = get_session(account, "org-script", region)
-        creds = session._session.get_credentials()
-    except ClientError:
-        log.error(
-            "unable to obtain credentials for account:%s role:%s",
-            account['name'], account['role'])
-        return 1
-
-    env = os.environ.copy()
+def _get_env_creds(session, region):
+    creds = session._session.get_credentials()
+    env = {}
     env['AWS_ACCESS_KEY_ID'] = creds.access_key
     env['AWS_SECRET_ACCESS_KEY'] = creds.secret_key
     env['AWS_SESSION_TOKEN'] = creds.token
     env['AWS_DEFAULT_REGION'] = region
+    return env
+
+
+def run_account_script(account, region, output_dir, debug, script_args):
+    try:
+        session = get_session(account, "org-script", region)
+    except ClientError:
+        return 1
+
+    env = os.environ.copy()
+    env.update(_get_env_creds(session, region))
 
     log.info("running script on account:%s region:%s script: `%s`",
              account['name'], region, " ".join(script_args))
@@ -489,9 +514,16 @@ def run_account(account, region, policies_config, output_path,
         account_id=account['account_id'], metrics_enabled=metrics,
         log_group=None, profile=None, external_id=None)
 
+    env_vars = account_tags(account)
+
     if account.get('role'):
-        config['assume_role'] = account['role']
-        config['external_id'] = account.get('external_id')
+        if isinstance(account['role'], six.string_types):
+            config['assume_role'] = account['role']
+            config['external_id'] = account.get('external_id')
+        else:
+            env_vars.update(
+                _get_env_creds(get_session(account, 'custodian', region), region))
+
     elif account.get('profile'):
         config['profile'] = account['profile']
 
@@ -500,11 +532,10 @@ def run_account(account, region, policies_config, output_path,
     success = True
     st = time.time()
 
-    with environ(**account_tags(account)):
+    with environ(**env_vars):
         for p in policies:
-
             # Variable expansion and non schema validation (not optional)
-            p.expand_variables(p.get_variables())
+            p.expand_variables(p.get_variables(account.get('vars', {})))
             p.validate()
 
             log.debug(
@@ -522,8 +553,8 @@ def run_account(account, region, policies_config, output_path,
             except ClientError as e:
                 success = False
                 if e.response['Error']['Code'] == 'AccessDenied':
-                    log.warning('Access denied account:%s region:%s',
-                                account['name'], region)
+                    log.warning('Access denied api:%s policy:%s account:%s region:%s',
+                                e.operation_name, p.name, account['name'], region)
                     return policy_counts, success
                 log.error(
                     "Exception running policy:%s account:%s region:%s error:%s",
@@ -562,17 +593,22 @@ def run_account(account, region, policies_config, output_path,
                   file_okay=False, dir_okay=True),
               default=None)
 @click.option("--metrics", default=False, is_flag=True)
+@click.option("--metrics-uri", default=None, help="Configure provider metrics target")
 @click.option("--dryrun", default=False, is_flag=True)
 @click.option('--debug', default=False, is_flag=True)
 @click.option('-v', '--verbose', default=False, help="Verbose", is_flag=True)
 def run(config, use, output_dir, accounts, tags, region,
-        policy, policy_tags, cache_period, cache_path, metrics, dryrun, debug, verbose):
+        policy, policy_tags, cache_period, cache_path, metrics,
+        dryrun, debug, verbose, metrics_uri):
     """run a custodian policy across accounts"""
     accounts_config, custodian_config, executor = init(
         config, use, debug, verbose, accounts, tags, policy, policy_tags=policy_tags)
     policy_counts = Counter()
     success = True
 
+    if metrics_uri:
+        metrics = metrics_uri
+
     if not cache_path:
         cache_path = os.path.expanduser("~/.cache/c7n-org")
         if not os.path.exists(cache_path):
